
local format in string

local _ENV = _G


-- set up Design.name2sys, Design.mount_mod_conflicts and
-- Design.mount_allowed_types eventually, these values will
-- need to change whenever the current tech tree changes.

local function size_mult_fun(prop,mult)

  local function bonus(size)
    return math.round(mult*size)
  end


  local fun =  function(self,hull_spec,size) 
    hull_spec[prop]=hull_spec[prop]+bonus(size)
  end

  local rval={}

  function rval.bonus_string(size)
    return  format("+%d %s", bonus(size),prop )
  end

  setmetatable(rval,{__call=fun} )

  return rval
end


local ranks = {
  'siege','heavy_beam','heavy_kinetic','energy_torpedo', 'missile',
  'mirv','fighter','shuttle','torpedo',
  'beam','kinetic','anti_missile',
  'pd_beam','pd_kinetic',
  --''
  'capacitor',

  'construction',
  'shield',
  'blast_door','bulkheads','transporter','boarding_module',

  
  'scanner',

  'system','lab','shield_regenerator',
  'warp_net',
  'cargo',

  'armor','reactor'
}

local type_rank = {}

for i,v ; ranks
  type_rank[v]=i
end

insert(Design,..type_rank)

function WeaponInfo_Property.damage_rank(info)
  local damage_fun in info
  local specials= info.specials?

  if (specials["Shield Damage Only"] or specials["EMP"])
    return 0
  end

  return info.num_mult * (damage_fun?.average or 0)
end

-- helpers for creating an ordered list of module names
do
  local function strick_less(a,b)
    if(a<b) 
      return true
    elseif(b<a) 
      return false
    end
  end

  local function comp_by_equal(a,b,test)
    if(a==b) return end

    if(a==test)
      return true
    elseif(b==test)
      return false
    end

  end

  function _G.module_comp(name1,name2)
    local all_parts in Design

    local sys1, sys2 = PartInfo[name1], PartInfo[name2]



    local _TRACE_INFO = args_string(..name1,..name2,..sys1.type,..sys2.type)

    return lexical_compare(
      type_rank![sys1.type] , type_rank![sys2.type],
      --sys1.weapon_cost,sys2.weapon_cost,
      --sys1.system_cost,sys2.system_cost,
      name1,name2,
    )

  end
end

function part_research_cost(name)
  
  for t,info in pairs(TECHNOLOGY)

    for _,p in ipairs(info.modules?)
      if p==name
        return info.cost
      end
    end

  end

end




-- for system's that have a defined tier value (specifically,
-- armor, reactors, and shields) return the available part of
-- the given type having the highest  tier.
function Design.best_of_type(_ENV,type,available_parts)
  local min_tier,rval

  for p,info ; available_parts
    if info.type==type
      if info!.tier > (min_tier or -1)
        min_tier=info.tier
        rval=p
      end
    end
  end

  return rval
end


Design.mount_mod_conflicts = {
--[[
  ["Siege"] = set('Point Defense'),
  ["Turret"] = set("Heavy Mount"),
  ["Cannon"] = set("Point Defense"),
  --]]
}

local _empty in _ENV
setmetatable(Design.mount_mod_conflicts,{__index=function() return _empty end } )


  
hardpoint_descs = {

  Siege = new_desc_object([[
    Siege weapon hardpoints are are capable of channeling the enormous
    energies required by the largest weapon systems.  In addition to
    siege weaponry, siege mounts may also be used to equip heavy weapons or
    missiles.
    ]]),

  Heavy = new_desc_object [=[
    Heavy weapons take up twice as much space as standard weapons. 
    Medium weapons can be placed in heavy hardpoints, in which case,
    they'll be twin-linked.
  ]=],

  Medium = new_desc_object [=[
    Medium weapon mounts have a fixed 90 degree forward firing arc when
    mounting a standard weapon.  
  ]=],

  Turret = new_desc_object [=[
    Turrets are the most versatile hardpoint type.  They can mount
    medium weapons, defensive light weapons, or
    anti-missiles. Any weapon placed in a turret mount will have a 360
    degree field of fire. ]=] ,

  Busbar = new_desc_object [=[
    Busbar hardpoints are unique to the Orthin.  Each busbar is placed at a
    node in the massive power distribution network that runs throughout
    the Orthin hulls.  Mounting different components in the busbar slot can grant the ship
    different special abilities.
  ]=] ,

  Light = new_desc_object [=[
    Light weapon hardpoints are auto-tracking turrets suitable for
    point-defense installations.  Light weapons take up half as much space
    as medium weapons. ]=] ,


  ["Armor"] = new_desc_object [[
    Armor: Mounting external armor plating does not reduce the space available
    for weapons and systems.  However, choosing not to outfit a ship with any armor
    can significantly reduce its production costs.
  ]],

  ["Shield"] = new_desc_object([[
    Shield: Dedicated space inside the hull that 
    can be filled with shield generators.  
  ]]),

  ["System"] = new_desc_object([[
    System: Generic system hardpoints represent space inside the hull that 
    can be filled with a range of critical systems, including drives,
    power and extra munitions.  
  ]]),

  ["Station"] = new_desc_object([=[
    Larger version of the generic [[System | system]] hardpoint that 
    can be filled with a range of critical systems, including drives,
    power and extra munitions. Station hardpoints can also mount modules 
    carrying small craft, [[Construction Module | construction modules]], [[Fuel Depot | fuel depots]], etc.

  ]=]),

  --[==[
  ["Gun"] = new_desc_object([=[
    Gun: Basic gun mounts are a type of small, simple weapon hardpoint included on destroyers and
    some light cruisers. Gun mounts lack both the rapid tracking systems of turrets, and the 
    heavy support structure of cannons. 
    Beam weapons mounted on a Gun hardpoint cannont be given either [[Point Defense]] or [[Heavy Mount]] modifications.
  ]=]),
  --]==]

  ["Heavy"] = new_desc_object([=[
    Heavy weapon mounts are designed for large beam weapons. However, the
    hardpoint can also be fitted with external missile launchers,
    scanners, or supplementary shield generators.

  ]=]),

  --[==[
  ["Turret"] = new_desc_object([=[
    Turret:
    Any beam weapons mounted on a turret have a 360 degree field of fire. 
    Turret mounts can be repurposed to hold additional armor, or to act as control centers for 
    fighter squadrons.
    Weapons mounted on a Turret hardpoint cannot be given the
    [[Heavy Mount]] modification.
  ]=]),
  --]==]
    

  ["Missile"] = new_desc_object([[
    Missile:
    Missile only hardpoints are built into some specialized hull designs.  
    While inflexible, they typically allow very high launch rates. 
  ]]),

  ["Torpedo"] = new_desc_object [[
    Torpedo: Large missile-only hardpoints are built into some specialized hull
    designs, which can mount heavy missiles, or an increased number of normal missiles.
  ]],

  ['Hangar'] = new_desc_object([[
    Carrier Bay:
    Carrier bay hardpoints are unique to carrier hulls and some base types.
    Designed to carry strike fighters or bombers, they can also be used to
    carry ground assault forces.
    ]]),

  ["Docking"] = new_desc_object([[
    Docking Bay:
    Docking bay hardpoints provide basic supply and support facilities
    for small craft.  They are well suited to transporting ground assault
    forces; and in a pinch can also be converted to house small strike
    fighter squadrons. ]]),

  ["Berth"] = new_desc_object([[
    Berth:
    Berth hardpoints provide basic supply and support facilities
    for small craft.  They are well suited to transporting ground assault
    forces; and in a pinch can also be converted to house small strike
    fighter squadrons. ]]),

  ["Planet"] = new_desc_object([[
    Planet:
    These hardpoints are unique to planetary defense bases and can mount essentially any
    weapon system, including superheavy weapons.<br><br>Missile weapons mounted in Planet slots have unlimited ammunition. ]]),
  
  ["Transport"] = new_desc_object[=[
    Transport: These hardpoints can contain [[Cargo Pods | cargo pods]], allowing the vessel to transport cargo, colonists, or ground forces.
    ]=]

--    <br><br>Ground forces assaulting a planet from a cargo pod will suffer a combat penalty. ]=],
}

for k,info ; hardpoint_descs
  INFO[format("%s Hardpoint",k)]=info
end

INFO.Armor=new_desc_object [=[
    Armor plating is the ship's second defense against incoming fire.
    As long as the remaining armor is greater than the ship's maximum [[Hull | hull integrity]], 
    any damage will be absorbed by the armor.
    However, at lower values it offers less and less protection.
    ]=]

INFO.Bunkers=new_desc_object [=[
    Planetary defense centers are housed in deep bunkers.  This grants them an initial layer of protection, much like a starship's [[Armor|armor]].
    ]=]

INFO.Shields=new_desc_object [=[
    Energy shields are a starship's first line of defense.
    As long as any shield strength remains, all incoming damage will be
    applied to the shield, rather than the ship's [[Armor|armor]] or [[Hull|hull]].
    ]=]

INFO.Upkeep=new_desc_object [=[
  Most starships require <COIN> to keep in working condition.  Larger
  ships, or those with more energy-hungry systems, will incur higher
  upkeep costs.
]=]

INFO.Systems=new_desc_object [[
    Each weapon and system inside the ship maintains its own damage value.  A ship with damaged systems will often be able to limp through the remainder of a battle, but,
    some systems, like a warp core or munitions stockpile, have a chance of causing a catastrophic explosion when they are destroyed.  
    Such explosions will not only destroy the ship itself, they also cause significant damage to anything nearby.
    ]]

INFO.Hull=new_desc_object [[
    Any damage breaching the ship's armor is applied to its hull.
    If reduced to less than 30% hull integrity, the ship will be destroyed.
    When a ship receives hull damage there is also a chance of damaging or destroying some of 
    the ship's systems, or causing casualties among the crew.
    ]]

INFO.Structure=new_desc_object [=[
    As with damage done to a ship's [[Hull|hull]], 
    any structural damage taken by a planetary defense center has a chance of disabling
    some or all of the defensive systems.  When an attack reduces the center's
    structural integrity to 30% or less, the underground bunkers collapse, and the planet 
    becomes a passive spectator to the battle.
    ]=]

INFO.Speed = new_desc_object "A ship's speed determines how many tiles it can move in a turn."

INFO.Cost = new_desc_object [[

  The <WRENCH> cost to produce a ship represents the amount of productive
  capacity that must be put into assembling it.  Adding more
  sophisticated components may greatly increase a designs costs.

]]

INFO.Metal = new_desc_object [[

  The <METAL> cost to produce a ship represents the amount of raw
  materials needed to build it.  Unlike <WRENCH> costs, <METAL> may be
  gathered from anywhere in your empire.  Build more <MINE1>
  improvements if you find yourself running short on <METAL>.
  
]]

INFO.Power = new_desc_object [[

  Each weapon or system added to a ship design requires a certain amount of
  <DESIGN_POWER> power.  Mounting multiple copies of more
  power hungry systems may not be possible without also adding
  additional reactors. 
]]


INFO.Hazard = new_desc_object [[
    Most ships carry volatile munitions or other potential explosives.  
    An unlucky hit will detonate them, destroying the ship and severely damaging 
    anything nearby.  The ship's hazard rating shows the amount of damage that will be done 
    to any vessels immediately adjacent to such an explosion.]]

INFO.Tanks = new_desc_object [[
    These heavily armored vehicles form the core of any planetary invasion.  A hostile planet can be conquered
    if the number of tank units in orbit is greater than the population of the world.]]

INFO.Crew=new_desc_object [[
    Crew casualties can be caused by either hull damage or boarding actions.  
    As more crewmembers are incapacitated, the ship becomes increasingly vulnerable to capture attempts.
  ]]

INFO.Munitions=new_desc_object [=[
  A large munitions stockpile will allow the ship to fire more missiles
  salvos, however, increasing the munition supply will also increase the ship's [[Hazard|hazard rating]].
]=]

